package fyloz.trial.ColorRecipesExplorer.core;

import org.springframework.web.servlet.ModelAndView;

import javax.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ModelBuilder {

    private static String RESPONSE_PREFIX = "response.";
    private static String ERROR_ATTRIBUTE_NAME = "error";
    private static String SUCCESS_ATTRIBUTE_NAME = "success";

    private ModelAndView model;
    private Map<String, Object> modelAttributes;

    public ModelBuilder(String view) {
        this(new ModelAndView(view));
    }

    public ModelBuilder(ModelAndView model) {
        this.model = model == null ? new ModelAndView() : model;

        modelAttributes = new HashMap<>();
    }

    public ModelBuilder setView(String view) {
        model.setViewName(view);

        return this;
    }

    public ModelBuilder addAttribute(String attributeName, Object content) {
        modelAttributes.put(attributeName, content);

        return this;
    }

    public ModelBuilder addResponseCode(ResponseCode responseCode, String... parameters) {
        int requiredParametersNumber = responseCode.getParametersNumber();
        int givenParametersNumber = parameters.length;
        StringBuilder builder = new StringBuilder();

        if (requiredParametersNumber != givenParametersNumber) {
            throw new IllegalArgumentException(String.format("Mauvais nombre de paramètre dans un réponse de modèle: %s requis, %s fournis", requiredParametersNumber, givenParametersNumber));
        }

        // Construit le code de réponse
        builder.append(RESPONSE_PREFIX);
        builder.append(responseCode.getCode());

        // Ajoute les paramètres, si nécessaire
        if (requiredParametersNumber > 0) {
            builder.append("('");

            for (int i = 0; i < givenParametersNumber; i++) {
                builder.append(parameters[i]);

                if (i < givenParametersNumber - 1) {
                    builder.append("','");
                }
            }

            builder.append("')");
        }

        // Ajoute l'attribut dans le Map
        addAttribute(responseCode.getType() == ResponseCode.ResponseCodeType.ERROR ? ERROR_ATTRIBUTE_NAME : SUCCESS_ATTRIBUTE_NAME, builder.toString());
        addAttribute("arg1", "Bathia");
        addAttribute("arg2", "test");

        return this;
    }

    public ModelBuilder addData(ModelDataType modelDataType, @NotNull Object data) {
        Class modelDataTypeClass = modelDataType.getDataType();
        Class modelListDataTypeClass = modelDataType.getListDataType();
        Class givenDataTypeClass = data.getClass();

        // Vérifie le type de l'objet
        if (!modelDataTypeClass.equals(givenDataTypeClass)) {
            throw new IllegalArgumentException(String.format("L'objet passé en paramètre n'est pas du bon type. Requis: %s, fournis: %s", modelDataTypeClass.getName(), givenDataTypeClass.getName()));
        }

        // Si l'objet est une liste, vérifie qu'elle n'est pas vide et qu'elle est du bon type
        if (modelDataTypeClass.equals(List.class) && modelListDataTypeClass != null) {
            List listData = (List) data;

            if (listData.size() < 1) {
                throw new IllegalArgumentException("La liste passée en paramètre ne contient aucun élément");
            }

            Class givenListDataTypeClass = listData.get(0).getClass();
            if (givenDataTypeClass.equals(modelListDataTypeClass)) {
                throw new IllegalArgumentException(String.format("La liste passée en paramètre contient des éléments du mauvais type. Requis: %s, fournis: %s", modelListDataTypeClass.getName(), givenListDataTypeClass.getName()));
            }
        }

        // Ajoute l'attribut dans le Map
        addAttribute(modelDataType.getDataTypeName(), data);

        return this;
    }

    public ModelAndView build() {
        model.addAllObjects(modelAttributes);

        return model;
    }
}
