package fyloz.trial.ColorRecipesExplorer.web.controller.creators;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Material;
import fyloz.trial.ColorRecipesExplorer.model.MixType;
import fyloz.trial.ColorRecipesExplorer.model.Recipe;
import fyloz.trial.ColorRecipesExplorer.services.MaterialService;
import fyloz.trial.ColorRecipesExplorer.services.MixService;
import fyloz.trial.ColorRecipesExplorer.services.MixTypeService;
import fyloz.trial.ColorRecipesExplorer.services.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.servlet.ModelAndView;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import static fyloz.trial.ColorRecipesExplorer.core.utils.ControllerUtils.redirect;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.*;
import static fyloz.trial.ColorRecipesExplorer.web.StringBank.MIX_TYPE;
import static fyloz.trial.ColorRecipesExplorer.web.StringBank.RECIPE_ID;

@Controller
public class MixCreatorController {

    private MixService mixService;
    private RecipeService recipeService;
    private MaterialService materialService;
    private MixTypeService mixTypeService;

    @Autowired
    public MixCreatorController(MixService mixService, RecipeService recipeService, MaterialService materialService, MixTypeService mixTypeService) {
        this.mixService = mixService;
        this.recipeService = recipeService;
        this.materialService = materialService;
        this.mixTypeService = mixTypeService;
    }

    /**
     * Affiche la page de création d'un mélange.
     * Cette méthode requiert l'identifiant d'une recette dans l'URL.
     *
     * @param model    Le Model injecté par Thymeleaf
     * @param recipeID L'identifiant de la recette
     * @return La page à afficher.
     */
    @GetMapping(CREATOR_MIX_SPECIFIC)
    public ModelAndView showCreationPage(ModelAndView model, @PathVariable int recipeID) {
        ModelBuilder modelBuilder = new ModelBuilder(model)
                .setView(CREATOR_MIX);
        Recipe recipe = recipeService.getByID(recipeID);

        if (recipe == null) {
            return modelBuilder
                    .setView(redirect(EDITOR_RECIPE))
                    .build();
        }

        List<MixType> associatedMixTypes = recipeService.getAssociatedMixesTypes(recipe);
        // Récupère seulement les produits qui ne sont pas des types de mélange OU que ce type existe dans la recette
        List<Material> materials = materialService.getAll().stream().filter(m -> !m.isMixType() || associatedMixTypes.contains(mixTypeService.getByMaterial(m))).collect(Collectors.toList());

        return modelBuilder
                .addData(ModelDataType.RECIPE, recipe)
                .addData(ModelDataType.MATERIALS, materials)
                .addAttribute("materialsJson", materialService.asJson(materials))
                .build();
    }

    /**
     * Permet à l'utilisateur de créer un mélange.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - La recette n'existe pas
     * - Un des produits n'existe pas
     * - Une erreur est survenue lors de la sauvegarde du type de mélange
     * - Une erreur est survenue lors de la sauvegarde du mélange
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param form La formulaire du mélange entré par l'utilisateur
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_MIX, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView createMix(@RequestBody LinkedMultiValueMap<String, String> form) {
        String mixTypeName = form.get(MIX_TYPE).get(0);
        int recipeID = Integer.parseInt(form.get(RECIPE_ID).get(0));

        List<Integer> materials = new ArrayList<>();
        List<Float> quantities = new ArrayList<>();
        // Trie les valeurs du formulaire de création du mélange
        for (String key : form.keySet()) {
            LinkedList<String> value = (LinkedList) form.get(key);

            if (key.equals("product")) {
                value.forEach(m -> materials.add(Integer.parseInt(m)));
            } else if (key.equals("inventoryQuantity")) {
                value.forEach(q -> quantities.add(Float.parseFloat(q)));
            }
        }

        ModelBuilder modelBuilder = new ModelBuilder(redirect(EDITOR_RECIPE_SPECIFIC.replaceAll("\\{" + RECIPE_ID + "}", String.valueOf(recipeID))));

        Recipe recipe = recipeService.getByID(recipeID);
        if (recipe == null) {
            return modelBuilder
                    .addResponseCode(ResponseCode.RECIPE_NOT_FOUND, String.valueOf(recipeID))
                    .build();
        }

        MixType mixType = mixTypeService.createByName(mixTypeName);
        if (mixType == null) {
            return modelBuilder
                    .setView(EDITOR_RECIPE)
                    .addResponseCode(ResponseCode.ERROR_SAVING)
                    .build();
        } else if (recipeService.getAssociatedMixesTypes(recipe).contains(mixType)) {
            return modelBuilder
                    .setView(EDITOR_RECIPE)
                    .addResponseCode(ResponseCode.MIX_TYPE_ALREADY_USED)
                    .build();
        }

        ModelBuilder creationResult = mixService.create(modelBuilder, materials, quantities, recipe, mixType);
        if (creationResult != null) {
            return showCreationPage(modelBuilder.build(), recipeID);
        }

        return modelBuilder.build();
    }
}
