package fyloz.trial.ColorRecipesExplorer.web.controller.editors;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Material;
import fyloz.trial.ColorRecipesExplorer.services.MaterialService;
import fyloz.trial.ColorRecipesExplorer.services.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.util.stream.Collectors;

import static fyloz.trial.ColorRecipesExplorer.core.utils.ControllerUtils.redirect;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.*;

@Controller
public class MaterialEditorController {

    private MaterialService materialService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialEditorController(MaterialService materialService, MaterialTypeService materialTypeService) {
        this.materialService = materialService;
        this.materialTypeService = materialTypeService;
    }

    /**
     * Affiche la page de tous les produits.
     * <p>
     * Modèle de la page:
     * - materials: La liste de tous les produits
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_MATERIAL)
    public ModelAndView listMaterials(ModelAndView model) {
        return new ModelBuilder(model)
                .setView(EDITOR_MATERIAL)
                .addData(ModelDataType.MATERIALS, materialService.getAll().stream().filter(m -> !m.isMixType()).collect(Collectors.toList()))
                .build();
    }

    /**
     * Affiche la page d'édition d'un produit.
     * La méthode requiert l'identifiant du produit à modifier dans l'URL.
     *
     * @param model      Le Model injecté par Thymeleaf
     * @param materialID L'identifiant du produit à modifier
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_MATERIAL_SPECIFIC)
    public ModelAndView showEditPage(ModelAndView model, @PathVariable int materialID) {
        ModelBuilder modelBuilder = new ModelBuilder(model)
                .setView(EDITOR_MATERIAL_EDITOR);
        Material material = materialService.getByID(materialID);

        if (material == null) {
            modelBuilder.addResponseCode(ResponseCode.MATERIAL_NOT_FOUND, String.valueOf(materialID));

            return listMaterials(modelBuilder.build());
        }

        return modelBuilder
                .addData(ModelDataType.MATERIAL, material)
                .addData(ModelDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    /**
     * Affiche la page d'upload d'un fichier SIMDUT pour un produit
     * Cette méthode requiert l'identifiant du produit dans l'URL
     * <p>
     * Modèle de la page:
     * - materialID: Contient l'identifiant du produit
     *
     * @param model      Le Model injecté par Thymeleaf
     * @param materialID L'identifiant du produit à modifier le SIMDUT
     * @return La page à afficher.
     */
    @GetMapping(value = EDIT_MATERIAL_SIMDUT_SPECIFIC)
    public ModelAndView chooseSIMDUTFile(ModelAndView model, @PathVariable int materialID) {
        return new ModelBuilder(model)
                .setView(EDIT_MATERIAL_SIMDUT)
                .addData(ModelDataType.MATERIAL_ID, materialID)
                .build();
    }

    /**
     * Sauvegarde le fichier SIMDUT d'un produit.
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param materialID L'identifiant du produit
     * @param simdut     Le fichier SIMDUT uploadé
     * @return La page à afficher.
     */
    @PostMapping(value = EDIT_MATERIAL_SIMDUT, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ModelAndView saveSIMDUT(int materialID, MultipartFile simdut) {
        ModelBuilder modelBuilder = new ModelBuilder(redirect("material/editor/" + materialID));
        Material material = materialService.getByID(materialID);

        if (material == null) {
            modelBuilder.addResponseCode(ResponseCode.MATERIAL_NOT_FOUND, String.valueOf(materialID));

            return chooseSIMDUTFile(modelBuilder.build(), materialID);
        }

        if (!materialService.removeSimdut(material) || !materialService.addSimdut(simdut, material)) {
            modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING_SIMDUT);

            return chooseSIMDUTFile(modelBuilder.build(), materialID);
        }

        return modelBuilder.build();
    }

    /**
     * Permet à l'utilisateur de modifier un produit.
     * <p>
     * La modification échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Il n'y a aucun produit avec l'identifiant passé en paramètre
     * - Une erreur est survenue lors de la mise à jour dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - materialCode: Contient le code du produit mit à jour
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param material Le produit à mettre à jour
     * @return La page à afficher.
     */
    @PostMapping(value = EDITOR_MATERIAL, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView saveEditedMaterial(Material material) {
        ModelBuilder modelBuilder = new ModelBuilder("");
        int materialID = material.getMaterialID();

        if (materialService.getByID(materialID) == null) {
            modelBuilder.addResponseCode(ResponseCode.MATERIAL_NOT_FOUND, String.valueOf(materialID));
        }

        if ((material = materialService.update(material)) != null) {
            modelBuilder.addData(ModelDataType.MATERIAL_CODE, material.getMaterialCode());
        } else {
            modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING);

            return showEditPage(modelBuilder.build(), materialID);
        }

        return listMaterials(modelBuilder.build());
    }
}
