package fyloz.trial.ColorRecipesExplorer.web.controller.editors;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Material;
import fyloz.trial.ColorRecipesExplorer.model.Mix;
import fyloz.trial.ColorRecipesExplorer.model.MixType;
import fyloz.trial.ColorRecipesExplorer.services.MaterialService;
import fyloz.trial.ColorRecipesExplorer.services.MixService;
import fyloz.trial.ColorRecipesExplorer.services.MixTypeService;
import fyloz.trial.ColorRecipesExplorer.services.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.servlet.ModelAndView;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import static fyloz.trial.ColorRecipesExplorer.core.utils.ControllerUtils.redirect;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.*;
import static fyloz.trial.ColorRecipesExplorer.web.StringBank.MIX_ID;

@Controller
public class MixEditorController {

    private MixService mixService;
    private MaterialService materialService;
    private RecipeService recipeService;
    private MixTypeService mixTypeService;

    @Autowired
    public MixEditorController(MixService mixService, MaterialService materialService, RecipeService recipeService, MixTypeService mixTypeService) {
        this.mixService = mixService;
        this.materialService = materialService;
        this.recipeService = recipeService;
        this.mixTypeService = mixTypeService;
    }

    /**
     * Affiche la page d'édition d'un mélange.
     * Cette méthode requiert l'identifiant du mélange à modifier dans l'URL.
     *
     * @param model Le Model injecté par Thymeleaf
     * @param mixID L'identifiant du mélange à modifier
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_MIX_SPECIFIC)
    public ModelAndView showPage(ModelAndView model, @PathVariable int mixID) {
        ModelBuilder modelBuilder = new ModelBuilder(model)
                .setView(EDITOR_MIX_SPECIFIC.replaceAll("/\\{" + MIX_ID + "}", ""));
        Mix mix = mixService.getByID(mixID);

        // Renvoie l'utilisateur à la page d'édition des recettes si le mélange n'est pas trouvé
        if (mix == null) {
            return modelBuilder
                    .setView(redirect(EDITOR_RECIPE))
                    .build();
        }

        List<Material> materials = new ArrayList<>();
        List<MixType> associatedMixTypes = recipeService.getAssociatedMixesTypes(mix.getRecipe());
        for (Material material : materialService.getAll()) {
            if (!material.isMixType() || (!mix.getMixType().getMaterial().equals(material) && associatedMixTypes.contains(mixTypeService.getByMaterial(material)))) {
                materials.add(material);
            }
        }

        return modelBuilder
                .addData(ModelDataType.MIX, mix)
                .addData(ModelDataType.RECIPE_CODE, mix.getRecipe().getRecipeCode())
                .addAttribute("mixJson", materialService.asJson(mix))
                .addAttribute("materialsJson", materialService.asJson(materials))
                .build();
    }

    /**
     * Permet à l'utilisateur de modifier un mélange.
     * <p>
     * La mise à jour échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Le mélange n'existe pas
     * - Un des produits n'existe pas
     * - Une erreur est survenue lors de la mise à jour dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param form  Le formulaire du mélange entré par l'utilisateur
     * @return La page à afficher.
     */
    @PostMapping(value = EDITOR_MIX, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView saveMix(@RequestBody LinkedMultiValueMap<String, Object> form) {
        ModelBuilder modelBuilder = new ModelBuilder("");
        int mixID = Integer.parseInt((String) form.get(MIX_ID).get(0));

        Mix mix = mixService.getByID(mixID);
        if (mix == null) {
            modelBuilder.addResponseCode(ResponseCode.MIX_NOT_FOUND, String.valueOf(mixID));

            return showPage(modelBuilder.build(), mixID);
        }

        modelBuilder.setView(redirect(EDITOR_RECIPE_SPECIFIC.replace("{recipeID}", String.valueOf(mix.getRecipe().getRecipeID()))));

        List<Integer> materials = new ArrayList<>();
        List<Float> quantities = new ArrayList<>();

        for (String key : form.keySet()) {
            LinkedList<String> value = (LinkedList) form.get(key);
            // TODO Laisser ?
            assert value != null : "Une valeur du formulaire d'édition d'un mélange est nulle.";

            if (key.equals("product")) {
                value.forEach(m -> materials.add(Integer.parseInt(m)));
            } else if (key.equals("inventoryQuantity")) {
                value.forEach(q -> quantities.add(Float.parseFloat(q)));
            }
        }

        if (!mixService.edit(mix, materials, quantities)) {
            modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING);

            return showPage(modelBuilder.build(), mixID);
        }

        return modelBuilder.build();
    }

    @GetMapping(REMOVER_MIX_SPECIFIC)
    public ModelAndView deleteMix(@PathVariable int mixID) {
        ModelBuilder modelBuilder = new ModelBuilder("");
        Mix mix = mixService.getByID(mixID);

        if (mix == null) {
            modelBuilder.addResponseCode(ResponseCode.MIX_NOT_FOUND, String.valueOf(mixID));

            return showPage(modelBuilder.build(), mixID);
        }

        modelBuilder.setView(redirect(EDITOR_RECIPE_SPECIFIC.replace("{recipeID}", String.valueOf(mix.getRecipe().getRecipeID()))));

        if (!mixService.deleteMix(mix)) {
            modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING);

            return showPage(modelBuilder.build(), mixID);
        }

        return modelBuilder.build();
    }
}
