package fyloz.trial.ColorRecipesExplorer.web.controller.editors;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Recipe;
import fyloz.trial.ColorRecipesExplorer.services.CompanyService;
import fyloz.trial.ColorRecipesExplorer.services.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.ArrayList;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.*;

@Controller
public class RecipeEditorController {

    private RecipeService recipeService;
    private CompanyService companyService;

    @Autowired
    public RecipeEditorController(RecipeService recipeService, CompanyService companyService) {
        this.recipeService = recipeService;
        this.companyService = companyService;
    }

    /**
     * Affiche la page listant toutes les recettes.
     * <p>
     * Modèle de la page:
     * - recipes: Contient un Map de toutes les recettes triées par compagnie
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_RECIPE)
    public ModelAndView listRecipes(ModelAndView model) {
        return new ModelBuilder(model)
                .addData(ModelDataType.RECIPE_MAP, recipeService.getRecipesByCompany())
                .build();
    }

    /**
     * Affiche la page d'édition d'une recette.
     * Cette méthode requiert l'identifiant de la recette dans l'URL.
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipe: Contient la recette
     * - recipeJSON: Contient la recette sous forme de JSON
     * - companies: Contient la liste des compagnies
     * - mixes: Contient la liste des mélanges de la recettes, classées
     * - images: Contient la liste des noms des images liées à la recette
     *
     * @param model    Le Model injecté par Thymeleaf
     * @param recipeID L'identifiant de la recette
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_RECIPE_SPECIFIC)
    public ModelAndView showEditPage(ModelAndView model, @PathVariable int recipeID) {
        ModelBuilder modelBuilder = new ModelBuilder(model)
                .setView(EDITOR_RECIPE_EDITOR);
        Recipe recipe = recipeService.getByID(recipeID);

        if (recipe == null) {
            modelBuilder.addResponseCode(ResponseCode.RECIPE_NOT_FOUND, String.valueOf(recipeID));

            return listRecipes(modelBuilder.build());
        }

        return modelBuilder
                .addData(ModelDataType.RECIPE, recipe)
                .addData(ModelDataType.COMPANIES, companyService.getAll())
                .addData(ModelDataType.MIXES, new ArrayList<>(recipeService.getSortedMixes(recipe)))    // Convertit le PersistentBag en ArrayList
                .addData(ModelDataType.IMAGES, recipeService.getImageFiles(recipe))
                .addAttribute("recipeJSON", recipeService.asJson(recipe))
                .build();
    }

    /**
     * Permet à l'utilisateur de modifier une recette.
     * <p>
     * La mise à jour échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - La recette n'existe pas
     * - Une erreur est survenue lors de la mise à jour dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipeCode: Contient la couleur d'une recette
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param recipe La recette à modifier
     * @param form   Le formulaire entré par l'utilisateur
     * @return La page à afficher.
     */
    @PostMapping(value = EDITOR_RECIPE, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView saveRecipe(@Valid Recipe recipe, @RequestBody MultiValueMap<String, Object> form) {
        ModelBuilder modelBuilder = new ModelBuilder("");
        int recipeID = recipe.getRecipeID();

        if (recipeService.getByID(recipeID) == null) {
            modelBuilder.addResponseCode(ResponseCode.RECIPE_NOT_FOUND, String.valueOf(recipeID));

            return listRecipes(modelBuilder.build());
        }

        recipe = recipeService.createAndSetSteps(recipe, form);

        modelBuilder.addData(ModelDataType.RECIPE_CODE, recipe.getRecipeCode());
        return listRecipes(modelBuilder.build());
    }
}
